package com.covas.Resources;

import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.HashSet;

import javax.annotation.security.PermitAll;
import javax.annotation.security.RolesAllowed;
import javax.inject.Inject;
import javax.ws.rs.CookieParam;
import javax.ws.rs.GET;
import javax.ws.rs.InternalServerErrorException;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.NewCookie;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.SecurityContext;

import com.covas.Classes.Hash;
import com.covas.Entity.UsersEntity;
import com.covas.Json.Jwt2;
import com.covas.Json.Token;

import io.smallrye.jwt.auth.principal.JWTParser;
import io.smallrye.jwt.auth.principal.ParseException;
import io.smallrye.jwt.build.Jwt;

import org.eclipse.microprofile.jwt.JsonWebToken;
import org.jboss.resteasy.annotations.jaxrs.HeaderParam;
import org.postgresql.shaded.com.ongres.scram.common.bouncycastle.base64.Base64;
import org.jboss.logging.Logger;

@Path("/token")
public class TokenRessource {

    private static final Logger LOGGER = Logger.getLogger(TokenRessource.class);

    @Inject
    JsonWebToken jwt;

   
    @Inject JWTParser parser;

    @GET
    @Path("authentificate")
    @Produces(MediaType.APPLICATION_JSON)
    public Response getUserName(@HeaderParam("Authorization") String auth, @CookieParam("jwt") String jwtCookie) {
        String name = "anonymous";
        String password = "";
        
        if (jwtCookie == null) {
            String[] hash = new String(Base64.decode(auth.split(" ")[1]), StandardCharsets.UTF_8).split(":");
            String pseudo = hash[0];
            password = Hash.encryptSHA512(hash[1]);

            UsersEntity users = UsersEntity.findByPseudo(pseudo);
            if(users != null){
                
            }

            
          // Create a JWT token signed using the 'HS256' algorithm
          String newJwtCookie = Jwt.issuer("https://example.com/issuer").upn("Alice").groups(new HashSet<>(Arrays.asList("User"))).sign();
          // or create a JWT token encrypted using the 'A256KW' algorithm
          // Jwt.upn("alice").encryptWithSecret(secret);
          
          return Response.status(Response.Status.CREATED).entity(new Jwt2(name, password)).cookie(new NewCookie("jwt", newJwtCookie)).build();
      } else {
          // All mp.jwt and smallrye.jwt properties are still effective, only the verification key is customized.
          try {
                jwt = parser.parse(jwtCookie);
          } 
          catch(ParseException p){
              return Response.status(Response.Status.NOT_ACCEPTABLE).entity(new Jwt2(name, false, p.getMessage())).build();
          }
          // or jwt = parser.decrypt(jwtCookie, secret);
          return Response.status(Response.Status.OK).entity(new Jwt2(jwt.getName(),password)).build();
        }
    }
    
    @GET
    @Path("permit-all")
    @PermitAll 
    @Produces(MediaType.APPLICATION_JSON)
    public Token hello(@Context SecurityContext ctx) {
        return getResponseString(ctx); 
    }

    @GET
    @Path("roles-allowed") 
    @RolesAllowed({"Admin" }) 
    @Produces(MediaType.APPLICATION_JSON)
    public Token helloRolesAllowed(@Context SecurityContext ctx) {
        Token token = getResponseString(ctx);
        token.name = jwt.getName().toString();
        token.role = "Admin";
        return token;
    }

    @GET
    @Path("roles-user") 
    @RolesAllowed({"Toto"}) 
    @Produces(MediaType.APPLICATION_JSON)
    public Token helloRolesUser(@Context SecurityContext ctx) {
        Token token = getResponseString(ctx);
        token.name = jwt.getName().toString();
        token.role = "User";
        return token;
    }


    private Token getResponseString(SecurityContext ctx) {
        String name;
        if (ctx.getUserPrincipal() == null) { 
            name = "anonymous";
        } else if (!ctx.getUserPrincipal().getName().equals(jwt.getName())) { 
            throw new InternalServerErrorException("Principal and JsonWebToken names do not match");
        } else {
            name = ctx.getUserPrincipal().getName(); 
        }
        return new Token(name, ctx.isSecure(), ctx.getAuthenticationScheme(), hasJwt());
    }

    private boolean hasJwt() {
	    return jwt.getClaimNames() != null;
    }
    
}
